@@ -2,7 +2,7 @@
  *  R : A Computer Language for Statistical Data Analysis
  *  file run.c: a simple 'reading' pipe (and a command executor)
  *  Copyright  (C) 1999-2001  Guido Masarotto and Brian Ripley
- *             (C) 2007-2021  The R Core Team
+ *             (C) 2007-2023  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ static char *expandcmd(const char *cmd, int whole)
     char c = '\0';
     char *s, *p, *q = NULL, *f, *dest, *src;
     int   d, ext, len = strlen(cmd)+1;
-    char buf[len], fl[len], fn[MAX_PATH];
+    char buf[len], fl[len+4], fn[MAX_PATH];
     DWORD res = 0;
 
     /* make a copy as we manipulate in place */
@@ -1031,7 +1031,9 @@ SEXP do_syswhich(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP nm, ans;
     int i, n;
+    const void *vmax = NULL;
 
+    vmax = vmaxget();
     checkArity(op, args);
     nm = CAR(args);
     if(!isString(nm))
@@ -1042,13 +1044,14 @@ SEXP do_syswhich(SEXP call, SEXP op, SEXP args, SEXP env)
 	if (STRING_ELT(nm, i) == NA_STRING) {
 	    SET_STRING_ELT(ans, i, NA_STRING);
 	} else {
-	    const char *this = CHAR(STRING_ELT(nm, i));
+	    const char *this = translateChar(STRING_ELT(nm, i));
 	    char *that = expandcmd(this, 1);
 	    SET_STRING_ELT(ans, i, mkChar(that ? that : ""));
 	    free(that);
 	}
     }
     setAttrib(ans, R_NamesSymbol, nm);
+    vmaxset(vmax);
     UNPROTECT(1);
     return ans;
 }
