@@ -842,7 +842,7 @@ Rd2HTML <-
     		switch(blocktag,
    		"\\value"=,
      		"\\arguments"= {
-    		    of1('<tr valign="top"><td>')
+    		    of1('<tr style="vertical-align: top;"><td>')
     		    inPara <<- NA
                     ## Argh.  Quite a few packages put the items in
                     ## their value section inside \code.
@@ -979,8 +979,8 @@ Rd2HTML <-
         if (done) return(FALSE)
         ## go through one by one until we hit \loadmathjax
         for (subfrag in frag) {
-            if (attr(subfrag, "Rd_tag") == "USERMACRO" &&
-                attr(subfrag, "macro") == "\\loadmathjax")
+            if (identical(attr(subfrag, "Rd_tag"), "USERMACRO") &&
+                identical(attr(subfrag, "macro"), "\\loadmathjax"))
                 return(TRUE)
         }
         return(FALSE)
@@ -1074,7 +1074,7 @@ Rd2HTML <-
 	    urlify(stylesheet),
 	    '" />\n',
 	    '</head><body><div class="container">\n\n',
-	    '<table width="100%">',
+	    '<table style="width: 100%;">',
             '<tr><td>',
             name)
 	if (nchar(package))
