@@ -68,7 +68,7 @@ function(x, ...)
         result <- c(result,
 		    paste0('<h2>', htmlify(x$title), ' in package &lsquo;',
 			   htmlify(pkg), '&rsquo;</h2>'),
-		    '<table cols="2" width="100%">',
+		    '<table cols="2" style="width: 100%;">',
 		    paste0('<tr>\n',
 			   ' <td style="text-align: left; vertical-align: top; width: 10%;">\n',
 			   htmlify(out[[pkg]][, "Item"]),
@@ -123,8 +123,11 @@ function(x, ...)
         vchunks[order(as.numeric_version(sub(" *patched", ".1",
                                              names(vchunks))),
                       decreasing = TRUE)]
-    vheaders <- sprintf("<h2>Changes in version %s</h2>",
-                        names(vchunks))
+    dates <- vapply(vchunks, function(v) v$Date[1L], "")
+    vheaders <- sprintf("<h2>Changes in version %s%s</h2>",
+                        names(vchunks),
+                        ifelse(is.na(dates), "",
+                               sprintf(" (%s)", dates)))
     c(HTMLheader(...),
       unlist(lapply(seq_along(vchunks),
                     function(i) {
@@ -167,7 +170,7 @@ function(x, ...)
                                              strict = FALSE),
                              decreasing = TRUE)]
 
-    dates <- sapply(vchunks, function(v) v$Date[1L])    
+    dates <- vapply(vchunks, function(v) v$Date[1L], "")
     vheaders <- sprintf("<h2>Changes in version %s%s</h2>",
                         names(vchunks),
                         ifelse(is.na(dates), "",
@@ -184,7 +187,7 @@ function(x, ...)
 # is depth 3
 
 makeVignetteTable <- function(vignettes, depth=2) {
-    out <- c('<table width="100%">',
+    out <- c('<table style="width: 100%;">',
              '<col style="width: 22%;" />',
              '<col style="width:  2%;" />',
              '<col style="width: 50%;" />',
@@ -203,13 +206,13 @@ makeVignetteTable <- function(vignettes, depth=2) {
 		  if (nchar(Outfile)) Outfile else File, '">',
 		  pkg, "::", topic, '</a>')
 	line <- c('<tr><td style="text-align: right; vertical-align: top;">', link,
-		    '</td>\n<td></td><td valign="top">', Title,
-		    '</td>\n<td valign="top">',
+		    '</td>\n<td></td><td style="vertical-align: top;">', Title,
+		    '</td>\n<td style="vertical-align: top;">',
 		    if (nchar(Outfile))
 			c('<a href="', root, Outfile,'">', vignette_type(Outfile), '</a>'),
-		    '</td>\n<td valign="top">',
+		    '</td>\n<td style="vertical-align: top;">',
 		    '<a href="', root, File,'">source</a>',
-		    '</td>\n<td valign="top" style="white-space: nowrap">',
+		    '</td>\n<td style="vertical-align: top; white-space: nowrap">',
 		    if (nchar(R))
 		    	c('<a href="', root, R,'">R code</a>'),
 		    '</td></tr>')
@@ -219,7 +222,7 @@ makeVignetteTable <- function(vignettes, depth=2) {
 }
 
 makeDemoTable <- function(demos, depth=2) {
-    out <- c('<table width="100%">',
+    out <- c('<table style="width: 100%;">',
              '<col style="width: 22%;" />',
              '<col style="width:  2%;" />',
              '<col style="width: 54%;" />',
@@ -242,8 +245,8 @@ makeDemoTable <- function(demos, depth=2) {
 	    runlink <- ""
 	}
 	line <- c('<tr><td style="text-align: right; vertical-align: top;">', link,
-		    '</td>\n<td></td><td valign="top">', Title,
-		    '</td>\n<td valign="top" style="white-space: nowrap">', runlink,
+		    '</td>\n<td></td><td style="vertical-align: top;">', Title,
+		    '</td>\n<td style="vertical-align: top; white-space: nowrap">', runlink,
 		    '</td></tr>')
 	out <- c(out, paste(line, collapse=''))
      }
@@ -251,7 +254,7 @@ makeDemoTable <- function(demos, depth=2) {
 }
 
 makeHelpTable <- function(help, depth=2) {
-    out <- c('<table width="100%">',
+    out <- c('<table style="width: 100%;">',
              '<col style="width: 22%;" />',
              '<col style="width:  2%;" />',
              '<col style="width: 74%;" />')
@@ -266,7 +269,7 @@ makeHelpTable <- function(help, depth=2) {
 		    ifelse(nchar(pkg), paste0(pkg, "::"), ""),
 		    topic, '</a>')
     lines <- paste0('<tr><td style="text-align: right; vertical-align: top;">', links,
-		    '</td>\n<td></td><td valign="top">', Title,
+		    '</td>\n<td></td><td style="vertical-align: top;">', Title,
 		    '</td></tr>')
     c(out, lines, '</table>')
 }
