@@ -1,7 +1,7 @@
 #  File src/library/utils/R/citation.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2018 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ function(given = NULL, family = NULL, middle = NULL,
                  email = email, role = role, comment = comment,
 		 first = first, last = last)
     if(all(vapply(args, is.null, NA)))
-        return(structure(list(), class = "person"))
+        return(.person())
 
     args <- lapply(args, .listify)
     args_length <- lengths(args)
@@ -58,21 +58,15 @@ function(given = NULL, family = NULL, middle = NULL,
                               sQuote("given"),
                               sQuote("first"), sQuote("middle")),
                      domain = NA)
-            ## <FIXME>
-            ## Start warning eventually ... maybe use message() for now?
-            message(gettextf("It is recommended to use %s instead of %s.",
+            warning(gettextf("It is recommended to use %s instead of %s.",
                              sQuote("given"), sQuote("first")),
                     domain = NA)
-            ## </FIXME>
             given <- first
         }
         if(!.is_not_nonempty_text(middle)) {
-            ## <FIXME>
-            ## Start warning eventually ... maybe use message() for now?
-            message(gettextf("It is recommended to use %s instead of %s.",
+            warning(gettextf("It is recommended to use %s instead of %s.",
                              sQuote("given"), sQuote("middle")),
                     domain = NA)
-            ## </FIXME>
             given <- c(given, unlist(strsplit(middle, "[[:space:]]+")))
         }
 
@@ -81,12 +75,9 @@ function(given = NULL, family = NULL, middle = NULL,
                 stop(gettextf("Use either %s or %s but not both.",
                               sQuote("family"), sQuote("last")),
                      domain = NA)
-            ## <FIXME>
-            ## Start warning eventually ... maybe use message() for now?
-            message(gettextf("It is recommended to use %s instead of %s.",
+            warning(gettextf("It is recommended to use %s instead of %s.",
                              sQuote("family"), sQuote("last")),
                     domain = NA)
-            ## </FIXME>
             family <- last
         }
 
@@ -117,8 +108,8 @@ function(given = NULL, family = NULL, middle = NULL,
         if(length(comment)) {
             ## Be nice and recognize ORCID identifiers given as URLs
             ## but perhaps without an ORCID name.
-            ind <- grepl(paste0("^https?://orcid.org/",
-                                "([[:digit:]]{4}[-]){3}[[:digit:]]{3}[[:alnum:]]$"),
+            ind <- grepl(sprintf("^https?://orcid.org/%s$",
+                                 tools:::.ORCID_iD_regexp),
                          comment)
             if(any(ind)) {
                 if(is.null(names(comment)))
@@ -155,8 +146,14 @@ function(given = NULL, family = NULL, middle = NULL,
     ## </COMMENT>
     ## Yes!
 
-    structure(rval[!vapply(rval, is.null, NA)],
-              class = "person")
+    .person(rval[!vapply(rval, is.null, NA)])
+}
+
+.person <-
+function(x = list())
+{
+    class(x) <- "person"
+    x
 }
 
 .canonicalize_person_role <-
@@ -264,12 +261,12 @@ c.person <-
 function(..., recursive = FALSE)
 {
     args <- list(...)
-    if(!all(sapply(args, inherits, "person")))
+    if(!all(vapply(args, inherits, NA, "person")))
         warning(gettextf("method is only applicable to %s objects",
                          sQuote("person")),
                 domain = NA)
     args <- lapply(args, unclass)
-    rval <- do.call("c", args)
+    rval <- do.call(c, args)
     class(rval) <- "person"
     rval
 }
@@ -322,9 +319,8 @@ function(x)
 
     ## Step B.
     pattern <- "[[:space:]]?(,|,?[[:space:]]and)[[:space:]]+"
-    x <- do.call("c",
-                 regmatches(x, gregexpr(pattern, y), invert = TRUE))
-    x <- x[!sapply(x, .is_not_nonempty_text)]
+    x <- do.call(c, regmatches(x, gregexpr(pattern, y), invert = TRUE))
+    x <- x[!vapply(x, .is_not_nonempty_text, NA)]
 
     ## don't expect Jr. to be a person
     jr <- which(!is.na(match(x, c("Jr", "Jr.", "jr", "jr."))))
@@ -382,18 +378,18 @@ function(x)
         return(z)
     }
 
-    as.list(do.call("c", lapply(x, as_person1)))
+    as.list(do.call(c, lapply(x, as_person1)))
 }
 
 personList <-
 function(...)
 {
     z <- list(...)
-    if(!all(sapply(z, inherits, "person")))
+    if(!all(vapply(z, inherits, NA, "person")))
         stop(gettextf("all arguments must be of class %s",
                       dQuote("person")),
              domain = NA)
-    do.call("c", z)
+    do.call(c, z)
 }
 
 as.personList <-
@@ -408,7 +404,7 @@ as.personList.default <-
 function(x)
 {
     if(inherits(x, "person")) return(x)
-    do.call("c", lapply(x, as.person))
+    do.call(c, lapply(x, as.person))
 }
 
 format.person <-
@@ -451,7 +447,7 @@ function(x,
     collapse <- lapply(collapse, rep_len, 1L)
 
     ## extract selected elements
-    x <- lapply(unclass(x), "[", include)
+    x <- lapply(unclass(x), `[`, include)
     braces <- braces[include]
     collapse <- collapse[include]
 
@@ -483,7 +479,7 @@ function(x,
                                                      collapse[[i]]),
                                       braces[[i]][2L])
                        })
-	paste(do.call("c", rval), collapse = " ")
+	paste(do.call(c, rval), collapse = " ")
     }
 
     sapply(x, format_person1)
@@ -507,13 +503,15 @@ function(object, ...)
 .canonicalize_ORCID_identifier <-
 function(x)
 {
-    paste0("https://orcid.org/", sub(".*/", "", x))
+    paste0("https://orcid.org/",
+           sub(tools:::.ORCID_iD_variants_regexp, "\\3", x))
 }
 
 .expand_ORCID_identifier <-
 function(x)
 {
-    if(any(ind <- (names(x) == "ORCID")))
+    if(any(ind <- ((names(x) == "ORCID") &
+                   grepl(tools:::.ORCID_iD_variants_regexp, x))))
         x[ind] <- paste0("<",
                          .canonicalize_ORCID_identifier(x[ind]),
                          ">")
@@ -531,8 +529,8 @@ function(bibtype, textVersion = NULL, header = NULL, footer = NULL, key = NULL,
 
     args <- c(list(...), other)
     if(!length(args))
-        return(structure(list(), class = "bibentry"))
-    if(any(sapply(names(args), .is_not_nonempty_text)))
+        return(.bibentry())
+    if(any(vapply(names(args), .is_not_nonempty_text, NA)))
         stop("all fields have to be named")
 
     ## arrange all arguments in lists of equal length
@@ -574,7 +572,7 @@ function(bibtype, textVersion = NULL, header = NULL, footer = NULL, key = NULL,
 
         ## process fields
         rval <- c(list(...), other)
-        rval <- rval[!sapply(rval, .is_not_nonempty_text)]
+        rval <- rval[!vapply(rval, .is_not_nonempty_text, NA)]
 	fields <- tolower(names(rval))
         names(rval) <- fields
         attr(rval, "bibtype") <- bibtype
@@ -607,8 +605,8 @@ function(bibtype, textVersion = NULL, header = NULL, footer = NULL, key = NULL,
     rval <- lapply(seq_along(args$bibtype),
                    function(i)
                    do.call(bibentry1,
-                           c(lapply(args, "[[", i),
-                             list(other = lapply(other, "[[", i)))))
+                           c(lapply(args, `[[`, i),
+                             list(other = lapply(other, `[[`, i)))))
 
     ## add main header/footer for overall bibentry vector
     if(!.is_not_nonempty_text(mheader))
@@ -616,8 +614,14 @@ function(bibtype, textVersion = NULL, header = NULL, footer = NULL, key = NULL,
     if(!.is_not_nonempty_text(mfooter))
         attr(rval, "mfooter") <- paste(mfooter, collapse = "\n")
 
-    class(rval) <- "bibentry"
-    rval
+    .bibentry(rval)
+}
+
+.bibentry <-
+function(x = list())
+{
+    class(x) <- "bibentry"
+    x
 }
 
 .bibentry_check_bibentry1 <-
@@ -630,7 +634,7 @@ function(x, force = FALSE)
         strsplit(tools:::BibTeX_entry_field_db[[bibtype]], "|",
                  fixed = TRUE)
     if(length(rfields) > 0L) {
-        ok <- sapply(rfields, function(f) any(f %in% fields))
+        ok <- vapply(rfields, function(f) any(f %in% fields), NA)
         if(any(!ok))
             stop(sprintf(ngettext(sum(!ok),
                                   "A bibentry of bibtype %s has to specify the field: %s",
@@ -726,10 +730,10 @@ function(x, style = "text", .bibstyle = NULL,
             Sys.getenv("_R_UTILS_FORMAT_BIBENTRY_VIA_RD_PERMISSIVE_",
                        "TRUE")
         permissive <- tools:::config_val_to_logical(permissive)
-        macros <- if(is.null(macros))
-		      tools:::initialRdMacros()
-                  else if(is.character(macros))
-		      tools::loadRdMacros(macros,
+        if(is.null(macros))
+            macros <- tools:::initialRdMacros()
+        else if(is.character(macros))
+            macros <- tools::loadRdMacros(macros,
                                           tools:::initialRdMacros())
         sapply(.bibentry_expand_crossrefs(x),
                function(y) {
@@ -746,7 +750,7 @@ function(x, style = "text", .bibstyle = NULL,
                                          permissive = permissive,
                                          macros = macros)
                    rd <- tools:::processRdSexprs(rd,
-                                                 "build",
+                                                 "install",
                                                  macros = attr(rd, "macros"))
                    f(rd, fragment = TRUE, out = out, ...)
                    paste(readLines(out), collapse = "\n")
@@ -754,21 +758,32 @@ function(x, style = "text", .bibstyle = NULL,
     }
 
     format_as_citation <- function(x, msg) {
-         c(paste(strwrap(attr(x, "mheader")), collapse = "\n"),
+        m <- attr(x, "mheader")
+        if(is.null(m) &&
+           is.null(unlist(x$header)) &&
+           !is.null(p <- attr(x, "package")))
+            m <- gettextf("To cite package %s in publications use:",
+                          sQuote(p))
+        i <- !is.null(m)
+        c(character(),
+          if(i)
+              paste(strwrap(m), collapse = "\n"),
           unlist(lapply(x, function(y) {
-              paste(c(if(!is.null(y$header))
-                      c(strwrap(y$header), ""),
-                      if(!is.null(y$textVersion)) {
-                          strwrap(y$textVersion, prefix = "  ")
-                      } else {
-                          format(y)
-                      },
+              h <- y$header
+              j <- !is.null(h)
+              s <- y$textVersion
+              if(is.null(s))
+                  s <- format(y)
+              paste(c(if(j)
+                          c(strwrap(h), ""),
+                      if(i || j)
+                          strwrap(s, prefix = "  ") else strwrap(s),
                       if(bibtex) {
                           c(gettext("\nA BibTeX entry for LaTeX users is\n"),
 			    paste0("  ", unclass(toBibtex(y))))
                       },
                       if(!is.null(y$footer))
-                      c("", strwrap(y$footer))),
+                          c("", strwrap(y$footer))),
                     collapse = "\n")
           })),
 	  paste(strwrap(c(attr(x, "mfooter"),
@@ -794,7 +809,7 @@ function(x, style = "text", .bibstyle = NULL,
                    out[!lengths(out)] <- ""
                    unlist(out)
                },
-               "citation" = format_as_citation(x,
+               "citation" = format_as_citation(.bibentry(x),
                                                msg = missing(bibtex) &&
                                                    missing(citation.bibtex.max)),
                "R" = .format_bibentry_as_R_code(x, ...)
@@ -841,7 +856,7 @@ function(x, more = list())
                          function(e)
                          tryCatch(.bibentry_check_bibentry1(e, TRUE),
                                   error = identity))
-        bad <- which(sapply(status, inherits, "error"))
+        bad <- which(vapply(status, inherits, NA, "error"))
         if(length(bad)) {
             for(b in bad) {
                 warning(gettextf("Dropping invalid entry %d:\n%s",
@@ -852,8 +867,7 @@ function(x, more = list())
         }
     }
 
-    class(x) <- "bibentry"
-    x
+    .bibentry(x)
 }
 
 print.bibentry <-
@@ -940,7 +954,7 @@ function(x, collapse = FALSE)
     s <- lapply(unclass(x),
                 function(e) {
                     a <- Filter(length, attributes(e)[anames])
-                    e <- e[!sapply(e, is.null)]
+                    e <- e[!vapply(e, is.null, NA)]
                     ind <- !is.na(match(names(e),
                                        c(anames, manames, "other")))
                     if(any(ind)) {
@@ -982,7 +996,7 @@ function(x)
 {
     s <- lapply(unclass(x),
                 function(e) {
-                    e <- e[!sapply(e, is.null)]
+                    e <- e[!vapply(e, is.null, NA)]
                     cargs <-
                         sprintf("%s = %s", names(e), sapply(e, deparse))
                     .format_call_RR("person", cargs)
@@ -1005,7 +1019,7 @@ function(x, name)
     ## </COMMENT>
     is_attribute <- name %in% bibentry_attribute_names
     rval <- if(is_attribute) lapply(unclass(x), attr, name)
-        else lapply(unclass(x), "[[", name)
+        else lapply(unclass(x), `[[`, tolower(name))
     if(length(rval) == 1L) rval <- rval[[1L]]
     rval
 }
@@ -1016,7 +1030,7 @@ function(x, name, value)
     is_attribute <- name %in% bibentry_attribute_names
 
     x <- unclass(x)
-    name <- tolower(name)
+    if(!is_attribute) name <- tolower(name)
 
     ## recycle value
     value <- rep_len(.listify(value), length(x))
@@ -1053,22 +1067,24 @@ function(x, name, value)
     ## check whether all elements still have their required fields
     for(i in seq_along(x)) .bibentry_check_bibentry1(x[[i]])
 
-    class(x) <- "bibentry"
-    x
+    .bibentry(x)
 }
 
+`$<-.citation` <-
+function(x, name, value)
+    .citation(NextMethod("$<-"), attr(x, "package"))
+
 c.bibentry <-
 function(..., recursive = FALSE)
 {
     args <- list(...)
-    if(!all(sapply(args, inherits, "bibentry")))
+    if(!all(vapply(args, inherits, NA, "bibentry")))
         warning(gettextf("method is only applicable to %s objects",
                          sQuote("bibentry")),
                 domain = NA)
     args <- lapply(args, unclass)
-    rval <- do.call("c", args)
-    class(rval) <- "bibentry"
-    rval
+    rval <- do.call(c, args)
+    .bibentry(rval)
 }
 
 toBibtex.bibentry <-
@@ -1076,7 +1092,9 @@ function(object, ...)
 {
     format_author <- function(author) paste(sapply(author, function(p) {
 	fnms <- p$family
-	only_given_or_family <- is.null(fnms) || is.null(p$given)
+	only_given_or_family <-
+            (is.null(fnms) || is.null(p$given)) &&
+            !(identical(fnms, "others") || identical(p$given, "others"))
 	fbrc <- if(length(fnms) > 1L ||
                    any(grepl("[[:space:]]", fnms)) ||
                    only_given_or_family) c("{", "}") else ""
@@ -1112,7 +1130,7 @@ function(object, ...)
 sort.bibentry <-
 function(x, decreasing = FALSE, .bibstyle = NULL, drop = FALSE, ...)
 {
-    x[order(tools::bibstyle(.bibstyle)$sortKeys(x),
+    x[order(tools::bibstyle(.bibstyle, .default = FALSE)$sortKeys(x),
             decreasing = decreasing),
       drop = drop]
 }
@@ -1179,10 +1197,11 @@ function(file, meta = NULL)
             mfooter <- c(mfooter, x)
     }
 
-    rval <- if(length(rval) == 1L)
-        rval[[1L]]
-    else
-        do.call("c", rval)
+    rlen <- length(rval)
+    if(rlen == 1L)
+        rval <- rval[[1L]]
+    else if(rlen > 1L)
+        rval <- do.call(c, rval)
     if(!.is_not_nonempty_text(mheader))
         attr(rval, "mheader") <- paste(mheader, collapse = "\n")
     if(!.is_not_nonempty_text(mfooter))
@@ -1200,8 +1219,8 @@ function(package = "base", lib.loc = NULL, auto = NULL)
     ## auto-generated package citation.
     if(!is.null(auto) &&
        !is.logical(auto) &&
-       !any(is.na(match(c("Package", "Version", "Title"),
-                        names(meta <- as.list(auto))))) &&
+       !anyNA(match(c("Package", "Version", "Title"),
+                    names(meta <- as.list(auto)))) &&
        !all(is.na(match(c("Authors@R", "Author"),
                         names(meta))))
        ) {
@@ -1211,8 +1230,7 @@ function(package = "base", lib.loc = NULL, auto = NULL)
         auto_was_meta <- FALSE
         dir <- system.file(package = package, lib.loc = lib.loc)
         if(dir == "")
-            stop(gettextf("package %s not found", sQuote(package)),
-                 domain = NA)
+            stop(packageNotFoundError(package, lib.loc, sys.call()))
         meta <- packageDescription(pkg = package,
                                    lib.loc = dirname(dir))
         ## if(is.null(auto)): Use default auto-citation if no CITATION
@@ -1347,9 +1365,6 @@ function(package = "base", lib.loc = NULL, auto = NULL)
                         tail(author, 1L), sep = " and ")
 
     rval <- bibentry(bibtype = "Manual",
-                     textVersion =
-                     paste0(author, " (", z$year, "). ", z$title, ". ",
-                            z$note, ". ", z$url),
                      header = header,
                      footer = footer,
                      other = z
@@ -1359,7 +1374,11 @@ function(package = "base", lib.loc = NULL, auto = NULL)
 
 .citation <-
 function(x, package = NULL)
-    structure(x, package = package, class = c("citation", "bibentry"))
+{
+    class(x) <- c("citation", "bibentry")
+    attr(x, "package") <- package
+    x
+}
 
 .read_authors_at_R_field <-
 function(x)
@@ -1370,13 +1389,13 @@ function(x)
         writeLines(x, con, useBytes = TRUE)
         eval(parse(con, encoding = "UTF-8"))
     } else {
-        eval(parse(text = x))
+        eval(str2expression(x))
     }
 
     ## Let's by nice ...
     ## Alternatively, we could throw an error.
     if(!inherits(out, "person"))
-        out <- do.call("c", lapply(x, as.person))
+        out <- do.call(c, lapply(x, as.person))
 
     out
 }
@@ -1480,7 +1499,7 @@ function(x)
     ## (RFC 5322 <https://tools.ietf.org/html/rfc5322>).
     if(any(ind <- grepl(",", display))) {
         display[ind] <- sprintf("\"%s\"",
-                                gsub("\"", "\\\\\"", display[ind]))
+                                gsub("\"", "\\\"", display[ind], fixed=TRUE))
     }
     paste(display, address)
 }
