@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996, 1997 Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2015 The R Core Team
+ *  Copyright (C) 1998-2022 The R Core Team
  *
  *  This source code module:
  *  Copyright (C) 1997, 1998 Paul Murrell and Ross Ihaka
@@ -19,7 +19,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -584,8 +584,11 @@ static int StringMatch(SEXP expr, const char *aString)
 #define S_RADICAL	 214
 #define S_SUM		 229
 #define S_INTEGRAL	 242
+
+#define S_ANGLELEFT	 225
 #define S_BRACKETLEFTTP	 233
 #define S_BRACKETLEFTBT	 235
+#define S_ANGLERIGHT	 241
 #define S_BRACKETRIGHTTP 249
 #define S_BRACKETRIGHTBT 251
 
@@ -824,8 +827,8 @@ static int TranslatedSymbol(SEXP expr)
 	code == 0260			||   /* degree */
 	code == 0262			||   /* second */
 	code == 0266                    ||   /* partialdiff */
-	code == 0321                    ||   /* nabla */
-	0)
+	code == 0321                         /* nabla */
+	)
 	return code;
     else // not translated
 	return 0;
@@ -976,6 +979,7 @@ static BBOX RenderSymbolStr(const char *str, int draw, mathContext *mc,
 	    mbs_init(&mb_st);
 	    while (*s) {
 		wc = 0;
+		// FIXME this does not allow for surrogate pairs (implausible)
 		res = mbrtowc(&wc, s, MB_LEN_MAX, &mb_st);
 		if(res == -1) error("invalid multibyte string '%s'", s);
 		if (iswdigit(wc) && font != PlainFont) {
@@ -1088,6 +1092,7 @@ static BBOX RenderStr(const char *str, int draw, mathContext *mc,
 	    const char *p = str;
 	    mbstate_t mb_st;
 	    mbs_init(&mb_st);
+	    // FIXME this does not allow for surrogate pairs
 	    while ((used = Mbrtowc(&wc, p, n, &mb_st)) > 0) {
 		/* On Windows could have sign extension here */
 		glyphBBox = GlyphBBox((unsigned int) wc, gc, dd);
@@ -1263,6 +1268,7 @@ static BBOX RenderSpace(SEXP expr, int draw, mathContext *mc,
 }
 
 static SymTab BinTable[] = {
+    { "!",               041 },
     { "*",		 052 },	/* Binary Operators */
     { "+",		 053 },
     { "-",		 055 },
@@ -1934,10 +1940,14 @@ static int DelimCode(SEXP expr, SEXP head)
 	    code = S_BRACKETLEFTBT;
 	else if (NameMatch(head, "rfloor"))
 	    code = S_BRACKETRIGHTBT;
-	if (NameMatch(head, "lceil"))
+	else if (NameMatch(head, "lceil"))
 	    code = S_BRACKETLEFTTP;
 	else if (NameMatch(head, "rceil"))
 	    code = S_BRACKETRIGHTTP;
+	else if (NameMatch(head, "langle"))
+	    code = S_ANGLELEFT;
+	else if (NameMatch(head, "rangle"))
+	    code = S_ANGLERIGHT;
     }
     else if (StringAtom(head) && length(head) > 0) {
 	if (StringMatch(head, "|"))
