@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2018  The R Core Team.
+ *  Copyright (C) 1998-2021  The R Core Team.
  *  Copyright (C) 1995-1998  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -220,7 +220,6 @@ static SEXP rep2(SEXP s, SEXP ncopy)
 	    SEXP elt = lazy_duplicate(VECTOR_ELT(s, i)); \
 	    for (j = (R_xlen_t) it[i]; j > 0; j--) \
 		SET_VECTOR_ELT(a, n++, elt); \
-	    if (j > 1) ENSURE_NAMEDMAX(elt); \
 	} \
 	break; \
     case RAWSXP: \
@@ -347,9 +346,13 @@ SEXP attribute_hidden do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
     R_xlen_t nc;
     SEXP a;
 
+    /* DispatchOrEval internal generic: rep.int */
     if (DispatchOrEval(call, op, "rep.int", args, rho, &a, 0, 0))
       return(a);
 
+    if (DispatchOrEval(call, op, "rep", args, rho, &a, 0, 0))
+      return(a);
+
     if (!isVector(ncopy))
 	error(_("invalid type (%s) for '%s' (must be a vector)"),
 	      type2char(TYPEOF(ncopy)), "times");
@@ -407,11 +410,25 @@ SEXP attribute_hidden do_rep_len(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     checkArity(op, args);
 
+    /* DispatchOrEval internal generic: rep_len */
     if (DispatchOrEval(call, op, "rep_len", args, rho, &a, 0, 0))
       return(a);
 
     s = CAR(args);
 
+    if (isObject(s)) {
+	SEXP rep_call;
+	PROTECT(rep_call = shallow_duplicate(call));
+	SETCAR(rep_call, install("rep"));
+	SET_TAG(CDDR(rep_call), install("length.out"));
+	SET_TAG(CDR(args), install("length.out"));
+	if (DispatchOrEval(rep_call, op, "rep", args, rho, &a, 0, 0)) {
+	    UNPROTECT(1);
+	    return(a);
+	}
+	UNPROTECT(1);
+    }
+
     if (!isVector(s) && s != R_NilValue)
 	error(_("attempt to replicate non-vector"));
 
@@ -531,8 +548,9 @@ static SEXP rep4(SEXP x, SEXP times, R_xlen_t len, R_xlen_t each, R_xlen_t nt)
 	for(i = 0, k = 0, k2 = 0; i < lx; i++) {			\
 	    /*		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();*/ \
 	    for(j = 0, sum = 0; j < each; j++) sum += (R_xlen_t) itimes[k++]; \
+	    SEXP elt = lazy_duplicate(VECTOR_ELT(x, i));		\
 	    for(k3 = 0; k3 < sum; k3++) {				\
-		SET_VECTOR_ELT(a, k2++, VECTOR_ELT(x, i));		\
+		SET_VECTOR_ELT(a, k2++, elt);				\
 		if(k2 == len) goto done;				\
 	    }								\
 	}								\
@@ -587,7 +605,8 @@ static SEXP rep4(SEXP x, SEXP times, R_xlen_t len, R_xlen_t each, R_xlen_t nt)
 	case EXPRSXP:
 	    for(i = 0; i < len; i++) {
 //		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-		SET_VECTOR_ELT(a, i, VECTOR_ELT(x, (i/each) % lx));
+		SEXP elt = lazy_duplicate(VECTOR_ELT(x, (i/each) % lx));
+		SET_VECTOR_ELT(a, i, elt);
 	    }
 	    break;
 	case RAWSXP:
@@ -621,6 +640,7 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
     static SEXP do_rep_formals = NULL;
 
     /* includes factors, POSIX[cl]t, Date */
+    /* DispatchOrEval internal generic: rep */
     if (DispatchOrEval(call, op, "rep", args, rho, &ans, 0, 0))
 	return(ans);
 
@@ -635,7 +655,7 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
 	do_rep_formals = allocFormalsList5(install("x"), install("times"),
 					   install("length.out"),
 					   install("each"), R_DotsSymbol);
-    PROTECT(args = matchArgs(do_rep_formals, args, call));
+    PROTECT(args = matchArgs_NR(do_rep_formals, args, call));
 
     x = CAR(args);
     /* supported in R 2.15.x */
@@ -781,11 +801,10 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
 SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans = R_NilValue /* -Wall */, from, to, by, len, along;
-    int nargs = length(args), lf;
-    Rboolean One = (nargs == 1);
     R_xlen_t i, lout = NA_INTEGER;
-    static SEXP do_seq_formals = NULL;
+    Rboolean One = (length(args) == 1); // *before* messing with args ..
 
+    /* DispatchOrEval internal generic: seq.int */
     if (DispatchOrEval(call, op, "seq", args, rho, &ans, 0, 1))
 	return(ans);
 
@@ -793,11 +812,12 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
        We pretend this is
        seq(from, to, by, length.out, along.with, ...)
     */
+    static SEXP do_seq_formals = NULL;
     if (do_seq_formals == NULL)
 	do_seq_formals = allocFormalsList6(install("from"), install("to"),
 					   install("by"), install("length.out"),
 					   install("along.with"), R_DotsSymbol);
-    PROTECT(args = matchArgs(do_seq_formals, args, call));
+    PROTECT(args = matchArgs_NR(do_seq_formals, args, call));
 
     from = CAR(args); args = CDR(args);
     to   = CAR(args); args = CDR(args);
@@ -807,9 +827,8 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
     Rboolean
 	miss_from = (from == R_MissingArg),
 	miss_to   = (to   == R_MissingArg);
-
     if(One && !miss_from) {
-	lf = length(from);
+	int lf = length(from);
 	if(lf == 1 && (TYPEOF(from) == INTSXP || TYPEOF(from) == REALSXP)) {
 	    double rfrom = asReal(from);
 	    if (!R_FINITE(rfrom))
@@ -839,7 +858,7 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
     }
 
     if(lout == NA_INTEGER) {
-	double rfrom, rto, rby = asReal(by);
+	double rfrom, rto;
 	if(miss_from) rfrom = 1.0;
 	else {
 	    if(length(from) != 1) errorcall(call, _("'%s' must be of length 1"), "from");
@@ -856,15 +875,20 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	if(by == R_MissingArg)
 	    ans = seq_colon(rfrom, rto, call);
-	else {
+	else { // 'by' specified
 	    if(length(by) != 1) errorcall(call, _("'%s' must be of length 1"), "by");
 	    double del = rto - rfrom;
 	    if(del == 0.0 && rto == 0.0) {
 		ans = to; // is *not* missing in this case
 		goto done;
 	    }
-	    /* printf("from = %f, to = %f, by = %f\n", rfrom, rto, rby); */
-	    double n = del/rby;
+	    double n, rby = asReal(by);
+	    Rboolean finite_del = R_FINITE(del);
+	    if(finite_del) {
+		n = del/rby;
+	    } else { // overflow in  (to - from)  when both are finite
+		n = rto/rby - rfrom/rby;
+	    }
 	    if(!R_FINITE(n)) {
 		if(del == 0.0 && rby == 0.0) {
 		    ans = miss_from ? ScalarReal(rfrom) : from;
@@ -872,8 +896,8 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		} else
 		    errorcall(call, _("invalid '(to - from)/by'"));
 	    }
-	    double dd = fabs(del)/fmax2(fabs(rto), fabs(rfrom));
-	    if(dd < 100 * DBL_EPSILON) {
+	    // inherited from seq.default() but "fudgy"
+	    if(finite_del && fabs(del)/fmax2(fabs(rto), fabs(rfrom)) < 100 * DBL_EPSILON) {
 		ans = miss_from ? ScalarReal(rfrom) : from;
 		goto done;
 	    }
@@ -888,8 +912,8 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 	    R_xlen_t nn;
 	    if((miss_from || TYPEOF(from) == INTSXP) &&
-	       (miss_to   || TYPEOF(to)   == INTSXP) &&
-	       TYPEOF(by) == INTSXP) {
+	       (miss_to   || TYPEOF(to)   == INTSXP) && TYPEOF(by) == INTSXP)
+	    {
 		int *ia, ifrom = miss_from ? (int)rfrom : asInteger(from),
 		    iby = asInteger(by);
 		/* With the current limits on integers and FEPS
@@ -907,8 +931,15 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		nn = (int)(n + FEPS);
 		ans = allocVector(REALSXP, nn+1);
 		double *ra = REAL(ans);
-		for(i = 0; i <= nn; i++)
-		    ra[i] = rfrom + (double)i * rby;
+		if(finite_del)
+		    for(i = 0; i <= nn; i++)
+			ra[i] = rfrom + (double)i * rby;
+		else { // |from - to| is infinite, but n = (from-to)/by is not
+		    rfrom /= 4.;
+		    rby   /= 4.;
+		    for(i = 0; i <= nn; i++) // ldexp(Y, 2) := Y * 2^2 = 4 Y
+			ra[i] = ldexp(rfrom + (double)i * rby, 2);
+		}
 		/* Added in 2.9.0 */
 		if (nn > 0)
 		    if((rby > 0 && ra[nn] > rto) || (rby < 0 && ra[nn] < rto))
@@ -919,20 +950,27 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	ans = allocVector(INTSXP, 0);
     } else if (One) {
 	ans = seq_colon(1.0, (double)lout, call);
-    } else if (by == R_MissingArg) { // and  len := length.out  specified
+    } else if (by == R_MissingArg) { // and  len := length.out  specified, >= 1
 	double rfrom = asReal(from), rto = asReal(to), rby = 0; // -Wall
 	if(miss_to)   rto   = rfrom + (double)lout - 1;
 	if(miss_from) rfrom = rto   - (double)lout + 1;
 	if(!R_FINITE(rfrom)) errorcall(call, _("'%s' must be a finite number"), "from");
 	if(!R_FINITE(rto))   errorcall(call, _("'%s' must be a finite number"), "to");
-	if(lout > 2) rby = (rto - rfrom)/(double)(lout - 1);
+	Rboolean finite_del = 0;
+	if(lout > 2) { // only then, use 'by'
+	    double nint = (double)(lout - 1);
+	    if((finite_del = R_FINITE(rby = (rto - rfrom))))
+		rby /= nint;
+	    else // overflow in (to - from), nint >= 2  => finite 'by'
+		rby = (rto/nint - rfrom/nint);
+	}
 	if(rfrom <= INT_MAX && rfrom >= INT_MIN &&
 	   rto   <= INT_MAX && rto   >= INT_MIN &&
 	   rfrom == (int)rfrom &&
 	   (lout <= 1 || rto == (int)rto) &&
 	   (lout <= 2 || rby == (int)rby)) {
 	    ans = allocVector(INTSXP, lout);
-	    if(lout > 0) INTEGER(ans)[0] = (int)rfrom;
+	    INTEGER(ans)[0] = (int)rfrom;
 	    if(lout > 1) INTEGER(ans)[lout - 1] = (int)rto;
 	    if(lout > 2)
 		for(i = 1; i < lout-1; i++) {
@@ -940,26 +978,35 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		    INTEGER(ans)[i] = (int)(rfrom + (double)i*rby);
 		}
 	} else {
-	ans = allocVector(REALSXP, lout);
-	if(lout > 0) REAL(ans)[0] = rfrom;
-	if(lout > 1) REAL(ans)[lout - 1] = rto;
-	if(lout > 2) {
-	    rby = (rto - rfrom)/(double)(lout - 1);
-	    for(i = 1; i < lout-1; i++) {
-//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-		REAL(ans)[i] = rfrom + (double)i*rby;
+	    ans = allocVector(REALSXP, lout);
+	    REAL(ans)[0] = rfrom;
+	    if(lout > 1) REAL(ans)[lout - 1] = rto;
+	    if(lout > 2) {
+		if(finite_del)
+		    for(i = 1; i < lout-1; i++) {
+// 		        if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+			REAL(ans)[i] = rfrom + (double)i*rby;
+		    }
+		else {  // del:=(from - to) is infinite, but n = del/by is not
+		    rfrom /= 4.; // or ldexp(*, -2) for speed
+		    rby   /= 4.;
+		    for(i = 1; i < lout-1; i++) {
+// 		        if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+			REAL(ans)[i] = ldexp(rfrom + (double)i*rby, 2); // ldexp(y,2) = y * 4
+		    }
+		}
 	    }
 	}
-	}
     } else if (miss_to) {
 	double rfrom = asReal(from), rby = asReal(by), rto;
 	if(miss_from) rfrom = 1.0;
 	if(!R_FINITE(rfrom)) errorcall(call, _("'%s' must be a finite number"), "from");
 	if(!R_FINITE(rby))   errorcall(call, _("'%s' must be a finite number"), "by");
 	rto = rfrom + (double)(lout-1)*rby;
-	if(rby == (int)rby && rfrom == (int)rfrom
-	   && rfrom <= INT_MAX && rfrom >= INT_MIN
-	   &&  rto  <= INT_MAX &&  rto  >= INT_MIN) {
+	// avoid undefined behaviour by testing range before converting.
+	if(rfrom <= INT_MAX && rfrom >= INT_MIN
+	   &&  rto  <= INT_MAX &&  rto  >= INT_MIN
+	   && rby == (int)rby && rfrom == (int)rfrom) {
 	    ans = allocVector(INTSXP, lout);
 	    for(i = 0; i < lout; i++) {
 //		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
@@ -1064,3 +1111,66 @@ SEXP attribute_hidden do_seq_len(SEXP call, SEXP op, SEXP args, SEXP rho)
     else
 	return R_compact_intrange(1, len);
 }
+
+SEXP attribute_hidden do_sequence(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    R_xlen_t lengths_len, from_len, by_len, ans_len, i, i2, i3;
+    int from_elt, by_elt, length, j, k, *ans_elt;
+    const int *lengths_elt;
+    SEXP ans, lengths, from, by;
+
+    checkArity(op, args);
+
+    lengths = CAR(args);
+    if (!isInteger(lengths))
+	error(_("'lengths' is not of mode integer"));
+    from = CADR(args);
+    if (!isInteger(from))
+	error(_("'from' is not of mode integer"));
+    by = CADDR(args);
+    if (!isInteger(by))
+	error(_("'by' is not of mode integer"));
+
+    lengths_len = length(lengths);
+    from_len = length(from);
+    by_len = length(by);
+    if (lengths_len != 0) {
+	if (from_len == 0)
+	    error(_("'from' has length 0, but not 'lengths'"));
+	if (by_len == 0)
+	    error(_("'by' has length 0, but not 'lengths'"));
+    }
+    ans_len = 0;
+    lengths_elt = INTEGER(lengths);
+    for (i = 0; i < lengths_len; i++, lengths_elt++) {
+	length = *lengths_elt;
+	if (length == NA_INTEGER || length < 0)
+	    error(_("'lengths' must be a vector of non-negative integers"));
+	ans_len += length;
+    }
+    PROTECT(ans = allocVector(INTSXP, ans_len));
+    ans_elt = INTEGER(ans);
+    lengths_elt = INTEGER(lengths);
+    for (i = i2 = i3 = 0; i < lengths_len; i++, i2++, i3++, lengths_elt++) {
+	if (i2 >= from_len)
+	    i2 = 0; /* recycle */
+	if (i3 >= by_len)
+	    i3 = 0; /* recycle */
+	length = *lengths_elt;
+	from_elt = INTEGER(from)[i2];
+	if (length != 0 && from_elt == NA_INTEGER) {
+	    UNPROTECT(1);
+	    error(_("'from' contains NAs"));
+	}
+	by_elt = INTEGER(by)[i3];
+	if (length >= 2 && by_elt == NA_INTEGER) {
+	    UNPROTECT(1);
+	    error(_("'by' contains NAs"));
+	}
+	// int to = from_elt + (length - 1) * by_elt;
+	for (k = 0, j = from_elt; k < length; j += by_elt, k++)
+	    *(ans_elt++) = j;
+    }
+    UNPROTECT(1);
+    return ans;
+}
