@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C)  2001-2020   The R Core Team.
+ *  Copyright (C)  2001-2021   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@
 #define R_USE_SIGNALS 1
 #include <Defn.h>
 #include <Rconnections.h>
-#include <R_ext/R-ftp-http.h>
 #include "sock.h"
 #include <errno.h>
 
@@ -188,7 +187,7 @@ static size_t sock_write(const void *ptr, size_t size, size_t nitems,
 			 Rconnection con)
 {
     Rsockconn this = (Rsockconn)con->private;
-    ssize_t n = R_SockWrite(this->fd, ptr, (int)(size * nitems),
+    ssize_t n = R_SockWrite(this->fd, ptr, (size_t)(size * nitems),
 			    this->timeout)/((ssize_t)size);
     return n > 0 ? n : 0;
 }
