@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
+ *  Copyright (C) 2000--2020 The R Core Team
  *  Copyright (C) 1998       Ross Ihaka
- *  Copyright (C) 2000--2005 The R Development Core Team
  *  based on AS 111 (C) 1977 Royal Statistical Society
  *  and   on AS 241 (C) 1988 Royal Statistical Society
  *
@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  *
  *  SYNOPSIS
  *
@@ -59,7 +59,7 @@ double qnorm5(double p, double mu, double sigma, int lower_tail, int log_p)
 #endif
     R_Q_P01_boundaries(p, ML_NEGINF, ML_POSINF);
 
-    if(sigma  < 0)	ML_ERR_return_NAN;
+    if(sigma  < 0)	ML_WARN_return_NAN;
     if(sigma == 0)	return mu;
 
     p_ = R_DT_qIv(p);/* real lower_tail prob. p */
@@ -81,8 +81,8 @@ double qnorm5(double p, double mu, double sigma, int lower_tail, int log_p)
         (original fortran code used PARAMETER(..) for the coefficients
          and provided hash codes for checking them...)
 */
-    if (fabs(q) <= .425) {/* 0.075 <= p <= 0.925 */
-        r = .180625 - q * q;
+    if (fabs(q) <= .425) {/* |p~ - 0.5| <= .425  <==> 0.075 <= p~ <= 0.925 */
+        r = .180625 - q * q; // = .425^2 - q^2  >= 0
 	val =
             q * (((((((r * 2509.0809287301226727 +
                        33430.575583588128105) * r + 67265.770927008700853) * r +
@@ -94,22 +94,18 @@ double qnorm5(double p, double mu, double sigma, int lower_tail, int log_p)
                    21213.794301586595867) * r + 5394.1960214247511077) * r +
                  687.1870074920579083) * r + 42.313330701600911252) * r + 1.);
     }
-    else { /* closer than 0.075 from {0,1} boundary */
-
-	/* r = min(p, 1-p) < 0.075 */
-	if (q > 0)
-	    r = R_DT_CIv(p);/* 1-p */
-	else
-	    r = p_;/* = R_DT_Iv(p) ^=  p */
-
-	r = sqrt(- ((log_p &&
-		     ((lower_tail && q <= 0) || (!lower_tail && q > 0))) ?
-		    p : /* else */ log(r)));
-        /* r = sqrt(-log(r))  <==>  min(p, 1-p) = exp( - r^2 ) */
+    else { /* closer than 0.075 from {0,1} boundary :
+	    *  r := log(p~);  p~ = min(p, 1-p) < 0.075 :  */
+	if(log_p && ((lower_tail && q <= 0) || (!lower_tail && q > 0))) {
+	    r = p;
+	} else {
+	    r = log( (q > 0) ? R_DT_CIv(p) /* 1-p */ : p_ /* = R_DT_Iv(p) ^=  p */);
+	}
+	// r = sqrt( - log(min(p,1-p)) )  <==>  min(p, 1-p) = exp( - r^2 ) :
+        r = sqrt(-r);
 #ifdef DEBUG_qnorm
 	REprintf("\t close to 0 or 1: r = %7g\n", r);
 #endif
-
         if (r <= 5.) { /* <==> min(p,1-p) >= exp(-25) ~= 1.3888e-11 */
             r += -1.6;
             val = (((((((r * 7.7454501427834140764e-4 +
@@ -125,7 +121,12 @@ double qnorm5(double p, double mu, double sigma, int lower_tail, int log_p)
                      r + 1.6763848301838038494) * r +
                     2.05319162663775882187) * r + 1.);
         }
-        else { /* very close to  0 or 1 */
+        else if(r >= 816) { // p is *extremly* close to 0 or 1 - only possibly when log_p =TRUE
+	    // Using the asymptotical formula -- is *not* optimal but uniformly better than branch below
+	    val = r * M_SQRT2;
+        }
+	else { // p is very close to  0 or 1:  r > 5 <==> min(p,1-p) < exp(-25) = 1.3888..e-11
+	    // Wichura, p.478: minimax rational approx R_3(t) is for 5 <= t <= 27  (t :== r)
             r += -5.;
             val = (((((((r * 2.01033439929228813265e-7 +
                        2.71155556874348757815e-5) * r +
@@ -147,6 +148,3 @@ double qnorm5(double p, double mu, double sigma, int lower_tail, int log_p)
     }
     return mu + sigma * val;
 }
-
-
-
