@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2022  The R Core Team
+ *  Copyright (C) 1997--2023  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -660,6 +660,15 @@ static void readline_handler(char *line)
 {
     R_size_t buflen = rl_top->readline_len;
 
+#if defined(RL_READLINE_VERSION) && RL_READLINE_VERSION >= 0x0802
+    /* In this version of readline, rl_redisplay called from
+       popReadline -> rl_free_line_state (-> rl_clear_message -> rl_redisplay)
+       wipes the prompt and moves the cursor accordingly, causing
+       undesirable indentation here when the input is empty, but the
+       cursor has been moved to the next line.
+       As a work-around, clear the prompt. */
+    if (line && !line[0]) rl_set_prompt("");
+#endif
     popReadline();
 
     if ((rl_top->readline_eof = !line)) /* Yes, I don't mean ==...*/
@@ -1122,21 +1131,21 @@ void attribute_hidden Rstd_WriteConsoleEx(const char *buf, int len, int otype)
 
 	/* Indicate that input is coming from the console */
 
-void attribute_hidden Rstd_ResetConsole()
+void attribute_hidden Rstd_ResetConsole(void)
 {
 }
 
 
 	/* Stdio support to ensure the console file buffer is flushed */
 
-void attribute_hidden Rstd_FlushConsole()
+void attribute_hidden Rstd_FlushConsole(void)
 {
     /* fflush(stdin);  really work on Solaris on pipes */
 }
 
 	/* Reset stdin if the user types EOF on the console. */
 
-void attribute_hidden Rstd_ClearerrConsole()
+void attribute_hidden Rstd_ClearerrConsole(void)
 {
     clearerr(stdin);
 }
@@ -1164,27 +1173,6 @@ void attribute_hidden Rstd_Busy(int which)
    If ask = SA_SUICIDE, no save, no .Last, possibly other things.
  */
 
-/* Note: now there is R_unlink(), but rm -Rf may be optimized for specific
-   file-systems. Some file systems allow to delete directory trees without
-   explicit/synchronous traversal so that deletion appears to be very
-   fast (see e.g. zfs). */
-void R_CleanTempDir(void)
-{
-    char buf[1024];
-
-    if((Sys_TempDir)) {
-// Only __sun is neeed on Solaris >= 10 (2005).
-#if defined(__sun) || defined(sun)
-	/* On Solaris the working directory must be outside this one */
-	chdir(R_HomeDir());
-#endif
-	snprintf(buf, 1024, "rm -Rf %s", Sys_TempDir);
-	buf[1023] = '\0';
-	R_system(buf);
-    }
-}
-
-
 void attribute_hidden NORET Rstd_CleanUp(SA_TYPE saveact, int status, int runLast)
 {
     if(saveact == SA_DEFAULT) /* The normal case apart from R_Suicide */
@@ -1330,6 +1318,7 @@ Rstd_ShowFiles(int nfile,		/* number of files */
     */
 
 
+#include <ctype.h>  /* for isspace */
 
 int attribute_hidden Rstd_ChooseFile(int _new, char *buf, int len)
 {
